NAME	LEDShow1

; Displays predefined text strings on the LED display in one of two modes.
; The display mode can be changed at run time with the switch.
;
; The program may be interrupted by External Interrupt 1. This will cause the
; processor to display a string and enter a wait loop with interrupts disabled.
; Only reset will restore normal operation. This facility is provided so that
; the programmer can trigger an orderly shutdown before reprogramming the part.
;
; The LED display consists of four devices of four elements each,
; for a total display capacity of sixteen characters.
; The display devices are numbered zero to three, from the right.
; The display elements are numbered from zero to three, from the right.
; Character positions are numbered 1 to 16, from the right.


NDEVS		EQU	4		; number of devices
NELMS		EQU	4		; number of elements in each device
SPACE		EQU	20h		; blank


		DSEG AT 60h		; stack origin
stack:		DS	20h		; stack depth

SWITCH		BIT	p1.0		; display mode select input


		CSEG

		ORG	0000h		; power on/reset vector
		jmp	init

		ORG	0003h		; external interrupt 0 vector
		reti			; undefined

		ORG	000bh		; timer 0 overflow vector
		reti			; undefined

		ORG	0013h		; external interrupt 1 vector
		jmp	shutdown

		ORG	001bh		; timer 1 overflow vector
		reti			; undefined

		ORG	0023h		; serial I/O interrupt vector
		reti			; undefined


		ORG	30h		; begin constant data space
pgm_msg:	DB	'  PROGRAMMING', 0
usr_msg:	DB	'ATMEL AT89S8252 CMOS MICROCONTROLLER'
		DB	' WITH FLASH MEMORY AND SPI PORT', 0


		ORG	0100h		; begin code space
		USING	0
init:
		mov	sp, #(stack-1)	; initialize stack pointer
		setb	IT1		; ext 1 interrupt edge triggered
		mov	IE, #10000100b	; enable ext 1 and global interrupts
	m0:
		jb	SWITCH, m1	; check position of switch

		call	rotate_msg	; display message
		jmp	m0		; again
	m1:
		call	shift_msg	; display message

		mov	a, #3		; pause 3 sec between displays
		call	delay_sec	;
		jmp	m0		; again


shutdown:

	; Respond to interrupt generated by serial programmer.

		clr	ea		; prevent interrupts

		mov	dptr, #pgm_msg	; point to message
		call	show_string	; display message

		jmp	$		; wait for reset


show_string:

	; Display null-terminated string pointed to by DPTR. The string is
	; left-justified in the display. If the length of the string exceeds
	; the number of display positions the excess characters are ignored.

		call	clear_display	; begin by blanking display

		mov	b, #(NDEVS*NELMS)	; total display positions
	gs1:
		clr	a		; get char
		movc	a, @a+dptr	;
		jz	gs2		; done if string terminator

		call	put_char	; display char at position in B

		inc	dptr		; point to next char
		djnz	b, gs1		; done when last position is filled
	gs2:
		ret


clear_display:

	; Fill display with blanks.
	; All registers preserved.

		push	acc
		push	b

		mov	b, #(NDEVS*NELMS)	; total display positions
	c1:
		mov	a, #SPACE
		call	put_char	; write space char
		djnz	b, c1		; do all positions

		pop	b
		pop	acc
		ret


shift_msg:

	; Display null-terminated string. Each character in the string,
	; in turn, enters the display from the right and is moved quickly
	; through each element of the display to its final position.
	; The string may contain any number of characters, including none.
	; If the length of the string exceeds the number of display
	; positions, the excess characters are ignored.

		call	clear_display	; begin by blanking display

		mov	r5, #(NDEVS*NELMS)	; total display positions
		mov	dptr, #usr_msg	; point to message
	ps1:
		mov	b, #1		; first display position
	ps2:
		clr	a
		movc	a, @a+dptr	; get char
		jz	ps4		; done if string terminator

		call	put_char	; display char at position in B
		mov	a, #25		; 25 mS
		call	delay_ms	; delay so char can be seen

		mov	a, b		; set up for compare
		clr	c		; ready for subtraction
		subb	a, r5		; compare next position to final
		jnc	ps3		; jump if char is in final position

		mov	a, #SPACE
		call	put_char	; blank out char

		inc	b		; next position
		jmp	ps2
	ps3:
		inc	dptr		; point to next char
		djnz	r5, ps1		; final position for next char
	ps4:
		ret


rotate_msg:

	; Display null-terminated string. The string moves through the
	; display, from right to left, with the display acting as a window
	; onto the string. The string may contain any number of characters,
	; including none.

		mov	dptr, #usr_msg	; point to string
		clr	a		; get first char
		movc	a, @a+dptr	;
		jz	dd11		; blank display and exit if null string

		call	clear_display	; begin by blanking display

		; Phase I. Shift the string INTO the display from the
		; right until the first character is in the left-most
		; display element. If the string has fewer characters than
		; the display has elements, pad the balance with blanks.

		mov	r7, #0		; loop counter, one pass per element
	dd1:
		mov	dptr, #usr_msg	; point to string
		mov	b, r7		; character position
		inc	b		; adjust
	dd2:
		clr	a		; get next char
		movc	a, @a+dptr	;
		jz	dd3		; jump if string terminator

		call	put_char	; display char at position in B

		inc	dptr		; point to next char
		djnz	b, dd2		; loop until all positions written
		jmp	dd5		; next pass
	dd3:				; encountered end of string
		mov	a, #SPACE	; pad balance of display with blanks
		call	put_char	; display char at position in B
		djnz	b, dd3		; next position
	dd5:
		mov	a, #150		; 150 mS
		call	delay_ms	; delay so string can be seen

		inc	r7		; next pass
		cjne	r7, #(NDEVS*NELMS), dd1	; loop until all elements done

		; Phase II. Shift the string THROUGH the display from
		; the right until the last character is in the left-most
		; display element. If the string has fewer characters than
		; the display has elements, pad the balance with blanks.

		mov	dptr, #usr_msg	; point to string
		inc	dptr		; start with the second char
	dd6:
		clr	a		; get char
		movc	a, @a+dptr	;
		jz	dd11		; blank display and exit if string end

		push	dpl		; save string pointer
		push	dph
		mov	b, #(NDEVS*NELMS)	; total char positions
	dd7:
		clr	a		; get next char
		movc	a, @a+dptr	;
		jz	dd8		; jump if string terminator

		call	put_char	; display char at position in B

		inc	dptr		; point to next char
		djnz	b, dd7		; loop until all positions written
		jmp	dd10		; next pass
	dd8:				; encountered end of string
		mov	a, #SPACE	; pad balance of display with blanks
		call	put_char	; display char at position in B
		djnz	b, dd8		; next position
	dd10:
		pop	dph		; restore string pointer
		pop	dpl		;
		inc	dptr		; point to next char

		mov	a, #150		; 150 mS
		call	delay_ms	; delay so string can be seen
		jmp	dd6		; process next char
	dd11:
		call	clear_display	; blank display

		mov	a, #150		; 150 mS
		call	delay_ms	; delay
		ret


put_char:

	; Display character in A at position indicated in B.
	; All registers preserved.

		push	acc
		push	b

		mov	p0, a		; move character to output port

		; Calculate device and element from display position.

		mov	a, b		; position 1..n
		dec	a		; convert to 0..n-1
		mov	b, #NELMS	; elements per device
		div	ab		; A= device, B= element

		mov	p2, #0ffh	; clear display control port
	s0:
		cjne	a, #0, s1	; check device number
		mov	a, #00010000b	; device 0 select
		jmp	s5
	s1:
		cjne	a, #1, s2
		mov	a, #00100000b	; device 1 select
		jmp	s5
	s2:
		cjne	a, #2, s3
		mov	a, #01000000b	; device 2 select
		jmp	s5
	s3:
		cjne	a, #3, s4
		mov	a, #10000000b	; device 3 select
		jmp	s5
	s4:
		jmp	init		; undefined device, restart
	s5:
		orl	a, b		; add element selector
		xrl	a, #11110000b	; invert device selector
		mov	p2, a		; write strobe low

		orl	a, #11110000b	; reset device selector
		mov	p2, a		; write strobe high (latch data)

		pop	b
		pop	acc
		ret


delay_ms:

	; Delay for one millisecond times the value in the accumulator.

		push	acc
		push	b

		mov	b, #0
	dd:
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	acc, dd

		pop	b
		pop	acc
		ret


delay_sec:

	; Delay for one second times the value in the accumulator.

		push	acc
		push	b

		mov	b, a
	ddd:
		mov	a, #250
		call	delay_ms	; 250 mS
		call	delay_ms	; 500 mS
		call	delay_ms	; 750 mS
		call	delay_ms	; 1000 mS

		djnz	b, ddd

		pop	b
		pop	acc
		ret


		END
