NAME	AT89S8252_Programmer

; The programmer powers up with the control signals to the target AT89S8252
; inactive, allowing the program in the target to run normally. Upon receipt
; of the first valid data record, the programmer puts the target into write
; mode. The first and subsequent valid records are dissected as they are
; received and their data is written into the target. Receipt of a valid
; end-of-file record terminates programming and resets the target control
; signals, allowing the new program in the target to run.
;
; Each record received is checked for validity. If it is invalid,
; the receiver sends a NAK to the remote system and discards the record.
; Bad records are not programmed into the target AT89S8252. Valid records
; are programmed into the target AT89S8252 and verified. If verification
; succeeds, an ACK is sent to the remote system. If verification fails,
; the receiver sends CAN to abort the upload. Failure to verify is a fatal
; error. The target AT89S8252 will be left in program mode (held reset) so
; that the incomplete or invalid code which it contains cannot be executed.
;
; Incoming records must appear less than thirty seconds apart, or the line
; is dropped in preparation for the next call. If the remote system drops
; the line, the programmer will wait thirty seconds before resetting. Calls
; incoming during this time are ignored.
;
; The programmer manages five lines (SHUTDN*, RST, SCK, MOSI, MISO)
; which control the target AT89S8252 and four lines which handle the modem
; interface. The AT89S8252 control lines occupy bits of port one and the
; modem interface lines bits of port three, as defined in the EQUates.
;
; Procedures SHOUT (SHift OUT) and SHIN (SHift IN) manage the serial transfer
; of data between the programmer and the target AT89S8252. The serial clock
; is generated and timed by software. The code meets timing requirements
; when executed by an AT89Cx051 microcontroller with a 12 MHz clock.
; Code modifications may be required if a faster clock is substituted.
;
; Two long period timers are implemented utilizing Timer Zero and members of
; register bank one. Timer Zero is configured in 16-bit mode and is loaded
; with an initial count of zero, which yields the maximum delay of 65.5 mS
; (at 12 MHz). The timer is allowed to free-run, generating an interrupt
; each time the count rolls over from FFFF to 0000. At each interrupt, the
; counts in each of the long period timers are decremented if their respective
; overflow flags are not set. If the new count in either long timer is zero,
; the corresponding overflow flag is set. It is not necessary to stop Timer
; Zero or to disable interrupts to reload the long timers, because they will
; not be disturbed by the Timer Zero interrupt service routine whenever their
; overflow flags are set. Because Timer Zero free-runs, it is not possible to
; know where in a period timing of an event begins. Therefore, one additional
; count should be added to the calculated long timer count to guarantee that
; the timed interval is not short.
;
; Long timer zero is sixteen bits, allowing a maximum timed interval of
; over one hour. Long timer one is eight bits, allowing a maximum timed
; interval of sixteen seconds.
;
; The programmer software is compatible with the U.S. Robotics Sportster
; 14,400 External modem and may require modifications if used with other
; modems. The switches on the modem are set as follows:
;
;	1	UP	DTR normal
;	2	DOWN	Numeric result codes
;	3	DOWN	Display result codes
;	4	DOWN	Suppress command echo
;	5	UP	Auto answer
;	6	UP	CD normal
;	7	UP	Load NVRAM defaults
;	8	DOWN	Smart mode
;
; Modem switch seven specifies that the power on and reset configuration be
; loaded from NVRAM profile zero, which must contain the factory default
; hardware flow control template. Other switch settings then over-ride the
; loaded configuration. If NVRAM profile zero does not contain the hardware
; flow control template, it may be restored with the following command
; sequence:
;
;	AT&F1&W0<ENTER>
;
; Some of the switch functions can be controlled by software, but making
; use of the switches simplifies the code required to initialize the modem.
; The only additional commands which must be issued to the modem are:
;
;	&R1	Ignore RTS,
;	&A0	Disable ARQ result codes.
;
; "&R1" causes the modem to forward incoming data to the programmer regardless
; of the state of RTS. "&A0" suppresses the extended protocol result codes.
; Note that suppression of the codes does not affect the connection. If it is
; desired to disable Error Control, issue the command "&M0".


CR		EQU	0dh		; carriage return
LF		EQU	0ah		; line feed

ACK		EQU	6h		; responses to remote system
NAK		EQU	15h		;
CAN		EQU	18h		;

BAUD_1200	EQU	0e8h		; 1200 baud timer reload values
BAUD_2400	EQU	0f4h		; 2400 baud
BAUD_9600	EQU	0fdh		; 9600 baud

OK		EQU	'0'		; modem status codes
RINGING		EQU	'2'		;
CONNECT_1200	EQU	'5'		;
CONNECT_2400	EQU	'10'		;
CONNECT_9600	EQU	'13'		;

MTRIES		EQU	5		; max attempts to access modem

ERASE_1		EQU	0ach		; erase chip function, first byte
ERASE_2		EQU	04h		; second byte
ENABLE_1	EQU	0ach		; enable write function, first byte
ENABLE_2	EQU	53h		; second byte
DUMMY		EQU	55h		; function third byte
WRITE_CODE	EQU	02h		; write code memory function (Flash)
READ_CODE	EQU	01h		; read code memory function
WRITE_DATA	EQU	06h		; write data memory function (EEPROM)
READ_DATA	EQU	05h		; read data memory function

lt0_lo		EQU	r2		; long timer one low byte
lt0_hi		EQU	r3		; long timer one high byte
lt1		EQU	r4		; long timer two only byte

index		EQU	r0		; general purpose index register
chksum		EQU	r5		; running checksum on record
temp		EQU	r6		; temporary storage
kount		EQU	r7		; loop counter

DSR_		BIT	p3.3		; modem control signals
DTR_		BIT	p3.7		;

RST		BIT	p1.7		; target control signals
SHUTDN_		BIT	p1.6		;
SCK		BIT	p1.4		;   serial clock
MOSI		BIT	p1.3		;   serial data out
MISO		BIT	p1.2		;   serial data in


		DSEG AT 20h

flags		DATA	20h		; misc flags
LT0F		BIT	flags.0		; long timer zero overflow flag
LT1F		BIT	flags.1		; long timer one overflow flag

		ORG	30h
rec_type:	DS	1		; record type
laddr_lo:	DS	1		; record load address, low byte
laddr_hi:	DS	1		; record load address, high byte
data_len:	DS	1		; record data byte count
data_buf:	DS	32		; storage for record data field

		ORG	60h		; stack origin
stack:		DS	20h		; stack depth

PCON		DATA	87h		; address of Power Control register
					;   (added to enlighten the assembler)

		CSEG

		ORG	0000h		; power on/reset vector
		jmp	init

		ORG	0003h		; external interrupt 0 vector
		reti			; undefined

		ORG	000Bh		; timer 0 overflow vector
		jmp	timer_int

		ORG	0013h		; external interrupt 1 vector
		reti			; undefined

		ORG	001Bh		; timer 1 overflow vector
		reti			; undefined

		ORG	0023h		; serial I/O interrupt vector
		jmp	serial_int


		ORG	40h		; begin constant data space

attn_cmd:	DB	'+++', 0	; modem return to command mode
reset_cmd:	DB	'ATZ', CR, 0	; modem reset string
					;   must be last command on line and
					;   modem returns code before executing
init_cmd:	DB	'AT&R1&A0', CR, 0	; modem init string
hangup_cmd:	DB	'ATH', CR, 0	; modem on-hook string


		ORG	0080h		; begin code space
		USING	0		; register bank zero
init:
		mov	sp, #(stack-1)	; initialize stack pointer
		call	initialize	; initialize controller registers

		setb	LT0F		; disable long timer zero
		setb	LT1F		; disable long timer one

		; Initialize the modem.

		setb	TI		; set transmit interrupt flag
					;   (kludge for first use)
		setb	ET0		; enable timer zero interrupt
		call	modem_init	; initialize modem
		clr	ET0		; disable timer zero interrupt
		jnc	m1		; jump if modem init passes

		clr	EA		; global interrupt disable
		orl	PCON, #1	; idle the controller, reset exits
	m1:
		; Clear pending interrupts before enabling serial interrupts.

		jnb	TI, $		; wait for transmitter to clear
		clr	TI		; clear transmit interrupt flag
		clr	RI		; clear receive interrupt flag
		setb	ES		; enable serial ints to wake controller
		clr	F0		; clear connect flag
	idle:
		orl	PCON, #1	; idle the controller, serial int exits
		jnb	F0, idle	; return to idle if not connected

		; Connection has been established.
		; Begin polling for valid record header.

		clr	ES		; disable serial interrupts
		setb	TI		; set transmit interrupt flag
					;   (kludge for first use)
		clr	F0		; clear program mode flag
		setb	ET0		; enable timer zero interrupt
	m2:
		call	init_longtimer0	; start 30 second timer
	m3:
		call	get_char	; get char, one second timeout
		jc	m8		; try again if parity error or timeout

		cjne	a, #':', m8	; try again if not record header

		; Found header, process hex record.

		call	get_record	; load and dissect record
		jnc	m4		; jump if record is good

		mov	a, #NAK		; tell sender record is bad
		call	send_char	;
		jmp	m2		; next record
	m4:
		cjne	a, #0, m6	; jump if record is not type zero

		; Process record type zero (data).

		jb	F0, m5		; jump if target is in write mode
		call	shutdown	; notify target of impending doom
;		call	erase_chip	; erase target
		call	set_pgm		; place target in write mode
		setb	F0		; flag target in write mode
	m5:
		call	write_record	; program data into target
		call	verify_record	; verify program data
		jnc	m7		; jump if verify OK

		mov	a, #CAN		; tell sender to abort
		call	send_char	;
		jmp	m9		; hang up and reset for next call
	m6:
		; Process record type one (end-of-file).

		call	clear_pgm	; take target out of write mode
		clr	F0		; flag target not in write mode
	m7:
		mov	a, #ACK		; tell sender record OK
		call	send_char	;
		jmp	m2		; next record
	m8:
		jnb	LT0F, m3	; poll until timer times out
	m9:				; timer timed out or upload cancelled
		call	hang_up		; break the connection
		clr	ET0		; disable timer zero interrupt
		jmp	m1		; return controller to idle


serial_int:

	; Process serial interrupt. Interrupts due to transmit done are
	; cleared and ignored. If interrupt is due to receive data ready,
	; check for a modem connect code, and set the connect flag.
	; The procedure includes code for identifying both single- and
	; double-character connect codes, but both may not be active
	; simultaneously. The code for identifying double-character
	; connect codes is dependent on the receive baud rate.
	; Serial interrupts are enabled elsewhere.

		clr	F0		; clear connect flag
		clr	TI		; clear transmit interrupt flag

		jnb	RI, si2		; exit if not receive data ready

		mov	a, SBUF		; get character into accumulator
		mov	c, p		; carry set for odd parity (error)
		jc	si1		; ignore char if parity error

		; Test for single-character 1200 baud connect code.

;;		anl	a, #7fh		; strip off parity (eighth) bit
;;		cjne	a, #CONNECT_1200, si1	; ignore char if wrong code

		; Test for double-character 9600 baud connect code.

		anl	a, #7fh		; strip off parity (eighth) bit
		cjne	a, #(HIGH CONNECT_9600), si1	; ignore wrong char

		clr	RI		; reset receive flag
		mov	a, #2		; expect next char in about one mS
		call	delay_ms	; wait for next char
		jnb	RI, si2		; exit if not receive data ready

		mov	a, SBUF		; get character into accumulator
		mov	c, p		; carry set for odd parity (error)
		jc	si1		; ignore char if parity error

		anl	a, #7fh		; strip off parity (eighth) bit
		cjne	a, #(LOW CONNECT_9600), si1	; ignore wrong char

		setb	F0		; set connect flag
	si1:
		clr	RI		; reset receive flag
	si2:
		reti


timer_int:

	; Process Timer Zero interrupt, which occurs about every 65.5 mS.
	; Each long timer count is decremented if its overflow flag is clear.
	; When a long timer count reaches zero, its overflow flag is set.
	; Counts are reloaded and overflow flags are reset elsewhere.

		push	psw		; save flags
		setb	RS0		; select register bank one

		jb	LT0F, ti2	; skip if long timer zero overflow set

		cjne	lt0_lo, #0, ti1	; test low byte
		dec	lt0_hi		; low byte is zero, borrow from high
	ti1:
		djnz	lt0_lo, ti2	; dec low byte, skip if not zero
		cjne	lt0_hi, #0, ti2	; low byte is zero, test high byte
					; both bytes equal zero
		setb	LT0F		; set overflow flag
	ti2:
		jb	LT1F, ti3	; skip if long timer one overflow set
		djnz	lt1, TI3	; decrement count and skip if not zero

		setb	LT1F		; count is zero, set overflow flag
	ti3:
		pop	psw		; restore flags and reg bank zero
		reti


initialize:

	; Initialize controller registers and I/O lines.

		mov	PCON, #0	; initialize power control register
		mov	IE, #0		; deactivate all interrupts

		mov	SCON, #01000000b	; serial port mode one
		mov	TMOD, #00100001b	; timer one 8-bit auto-reload,
						;   timer zero 16-bit
;;		mov	TH1, #BAUD_1200		; timer one reload value
		mov	TH1, #BAUD_9600		; timer one reload value
		mov	TCON, #01000000b	; start timer one

		mov	TL0, #0		; set timer zero to max count
		mov	TH0, #0		;
		setb	TR0		; start timer zero

		setb	REN		; enable serial reception
		setb	EA		; global interrupt enable

		; Initialize I/O lines.

		setb	DTR_
		setb	SHUTDN_
		setb	MISO
		setb	MOSI
		clr	SCK
		clr	RST		; remove reset from target
		ret


modem_init:

	; Reset and initialize the modem.
	; Return with carry set if modem fails to respond as expected.

		clr	DTR_		; assert DTR to talk to modem

		; First must ensure that the modem is in command mode.

		mov	a, #1		; wait one second
		call	delay_sec	;

		mov	dptr, #attn_cmd	; point to attention string
		call	send_string	; transmit string

		mov	a, #1		; wait one second
		call	delay_sec	;

		; Reset modem, causing the switches to be read.

		mov	dptr, #reset_cmd	; point to reset string
		call	modem_cmd		; transmit string
		jc	nn1			; jump on fail

		mov	a, #1		; wait one second before next command
		call	delay_sec	;

		; Modem is powered up and on-line.
		; Send required software parameters.

		mov	dptr, #init_cmd	; point to init string
		call	modem_cmd	; transmit string
		jnc	nn2		; jump on pass
	nn1:
		; Modem is misbehaving, so deactivate it.
		; The controller must be reset to exit this state.

		setb	DTR_		; deassert DTR to deactivate modem
	nn2:
		ret


hang_up:

	; Force the modem to drop the line.

		; First must return the modem to command mode.

		mov	a, #1		; wait one second
		call	delay_sec	;

		mov	dptr, #attn_cmd	; point to attention string
		call	send_string	; transmit string

		mov	a, #1		; wait one second
		call	delay_sec	;

		; Issue command to hang up.

		mov	dptr, #hangup_cmd	; point to hang up string
		call	modem_cmd		; transmit string
		jnc	hh			; jump on pass

		; The polite way didn't work, so drop DTR.
		; The controller must be reset to exit this state.

		setb	DTR_		; force modem to drop the line
	hh:
		ret


modem_cmd:

	; Transmit command string to modem and validate the response.
	; Return with carry set if modem fails to respond as expected,
	; or if excessive parity errors or receive timeouts occur.
	; Valid responses consist of a byte code followed by a carriage
	; return. Parity errors and timeouts cause the command to be
	; resent. Expected delays for command responses are absorbed
	; by GET_CHAR. On entry, DPTR must point to a null-terminated
	; command string.

		push	b
		mov	b, #MTRIES	; number of attempts
	mm1:
		call	send_string	; transmit command string
		clr	RI		; discard any waiting character
	mm2:
		call	get_char	; receive result code
		jc	mm3		; jump on parity error or timeout
		cjne	a, #OK, mm2	; loop if response is not valid

		call	get_char	; receive carriage return
		jc	mm3		; jump on parity error or timeout
		cjne	a, #CR, mm2	; loop if response is not valid
					; valid response complete
		clr	c		; clear error flag
		jmp	mm4		; return
	mm3:
		djnz	b, mm1		; resend command
		setb	c		; out of retries, set error flag
	mm4:
		pop	b
		ret


send_string:

	; Transmit string pointed to by DPTR.
	; String may be of any length, but must be null-terminated.

		push	acc
		push	dpl
		push	dph
	ss1:
		clr	a
		movc	a, @a+dptr	; get character
		jz	ss2		; check for terminator

		call	send_char	; send character

		inc	dptr		; point to next character
		jmp	ss1
	ss2:
		pop	dph
		pop	dpl
		pop	acc
		ret


send_char:

	; Wait for transmitter to clear, add even parity bit to character
	; in accumulator and transmit it. Does not wait for transmitter
	; to clear before returning.

		jnb	TI, $		; wait here for transmitter to clear
		clr	TI		; clear transmit flag
		push	acc		; save char
		mov	c, p		; get parity bit
		mov	acc.7, c	; add parity bit to data
		mov	SBUF, a		; load character into transmitter
		pop	acc		; restore char
		ret


get_char:

	; Read a character from the serial port and check for even parity.
	; Return the character in the accumulator with parity stripped off.
	; The routine will wait for approximately one second before timing
	; out. Return with carry set on parity error or timeout.

		jb	RI, gc2		; jump if char is waiting

		call	init_longtimer1	; start one second timer
	gc1:
		jb	RI, gc2		; exit loop when char received
		jnb	LT1F, gc1	; loop until timer times out

		setb	c		; set error flag
		jmp	gc3		; return
	gc2:
		mov	a, SBUF		; get character into accumulator
		mov	c, p		; carry set for odd parity (error)
		anl	a, #7fh		; strip off parity (eighth) bit
		clr	RI		; reset receive flag
	gc3:
		ret


get_byte:

	; Read two hexadecimal ASCII characters from the serial port
	; and return their binary equivalent in the accumulator.
	; Return with carry set if either character was invalid or
	; contained a parity error.

		call	get_char	; get first char from serial port
		jc	gb		; exit on parity error

		call	ascii2bin	; convert hex to binary
		jc	gb		; exit on invalid char

		swap	a		; first hex digit times sixteen
		mov	b, a		; save value

		call	get_char	; get second char from serial port
		jc	gb		; exit on parity error

		call	ascii2bin	; convert hex to binary
		jc	gb		; exit on invalid char

		orl	a, b		; combined binary equivalent
	gb:
		ret


ascii2bin:

	; Convert hexadecimal digit in the accumulator to its binary
	; equivalent and return it in the accumulator. Valid hex digits
	; are 0..9 and A..F (upper case only). Return with carry set
	; if the character received is not a valid hex digit.

		mov	temp, a		; save char
		clr	c		; prepare for subtraction
		subb	a, #('9'+1)	; compare to '9'
		jnc	a1		; jump if char above '9'

		mov	a, temp		; get original char
		clr	c		; prepare for subtraction
		subb	a, #'0'		; compare to '0'
		jmp	a4		; return error if char below '0'
					;   else binary value in accumulator
	a1:
		mov	a, temp		; get original char
		subb	a, #('F'+1)	; compare to 'F'
		cpl	c		; invert error flag
		jc	a4		; return error if char is above 'F'
	a2:
		mov	a, temp		; get original char
		subb	a, #'A'		; compare to 'A'
		jc	a4		; return error if char is below 'A'
	a3:
		add	a, #10		; adjust binary value
	a4:
		ret


get_record:

	; Read and dissect record. Two record types are accepted: data and
	; end-of-file. If the record type is data, the appropriate values
	; are extracted and stored. If the record type and checksum are
	; valid, the carry bit is cleared and the record type is returned
	; in the accumulator. Return with carry set to signal an invalid
	; record type, checksum error, or other problem. Errors returned
	; by routine GET_BYTE (invalid char or parity) cause an immediate
	; return with carry set.

		mov	chksum, #0	; clear running checksum

		call	get_byte	; get record data length field
		jc	rr4		; jump on error
		mov	data_len, a	; save data length

		clr	c		; prepare for subtraction
		subb	a, #(16+1)	; data length limited to 16 bytes
		jnc	rr4		; jump if max size exceeded

		call	get_byte	; get high byte of load address field
		jc	rr4		; jump on error
		mov	laddr_hi, a	; save it

		call	get_byte	; get low byte of load address field
		jc	rr4		; jump on error
		mov	laddr_lo, a	; save it

		call	get_byte	; get record type field
		jc	rr4		; jump on error
		mov	rec_type, a	; save type
		cjne	a, #0, rr2	; jump if not type zero (data)

		; Process data in data type record.

		mov	index, #data_buf	; pointer to data buffer
		mov	kount, data_len		; byte counter
	rr1:
		call	get_byte	; get data from serial port
		jc	rr4		; jump on error
		mov	@index, a	; save data in buffer

		add	a, chksum	; update checksum
		mov	chksum, a	;

		inc	index		; point to next location
		djnz	kount, rr1	; decrement byte count and loop
		jmp	rr3		; done with data, do checksum
	rr2:
		mov	a, rec_type	; get record type
		cjne	a, #1, rr4	; jump if not type one (end-of-file)
	rr3:
		; Process checksum.

		call	get_byte	; get record checksum
		jc	rr4		; jump on error
		add	a, chksum	; update running checksum
		add	a, data_len	;
		add	a, laddr_lo	;
		add	a, laddr_hi	;
		add	a, rec_type	;
		jnz	rr4		; jump if record checksum is not zero

		; Discard CR/LF which terminates record.

;		call	get_byte
;		jc	rr4		; jump on error
;		call	get_byte
;		jc	rr4		; jump on error

		mov	a, rec_type	; return record type in accumulator
		clr	c		; no errors
		jmp	rr5		; return
	rr4:
		; Error: data field too large, invalid type or bad checksum.

		setb	c		; set error flag
	rr5:
		ret


write_record:

	; Write the data extracted from the most recently received record
	; into the target AT89S8252. Timing delays are enforced by software.
	; This routine assumes that the target has already been prepared
	; for programming. Returns nothing.

		mov	r2, laddr_lo	; save low byte of load address
		mov	r3, laddr_hi	; save high byte of load address
		mov	a, r3		; get high byte of load address
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #WRITE_CODE	; specify code write function
		mov	temp, a		; save adjusted high byte

		mov	index, #data_buf	; pointer to data buffer
		mov	kount, data_len		; byte counter
	pp1:
		mov	a, temp		; send adjusted high byte of address
		call	shout		;
		mov	a, r2		; send low byte of address
		call	shout		;
		mov	a, @index	; send data from buffer
		call	shout		;

		mov	a, #3		; wait three milliseconds
		call	delay_ms

		; Next address.

		mov	a, r2		; get low byte of address
		add	a, #1		; increment low byte
		mov	r2, a		; save incremented value
		jnc	pp2		; jump if no carry out of low byte
					; carry out of low byte
		mov	a, r3		; get high byte of address
		add	a, #1		; increment high byte
		mov	r3, a		; save incremented value
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #WRITE_CODE	; specify code write function
		mov	temp, a		; save adjusted high byte
	pp2:
		; Next data.

		inc	index		; point to next buffer location
		djnz	kount, pp1	; decrement byte count and loop
		ret


verify_record:

	; Verify the data extracted from the latest record against that
	; written into the target AT89S8252. Timing delays are enforced by
	; software. This routine assumes that the target has already been
	; prepared for programming. Return with carry set if verify fails.

		mov	r2, laddr_lo	; save low byte of load address
		mov	r3, laddr_hi	; save high byte of load address
		mov	a, r3		; get high byte of load address
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #READ_CODE	; specify code read function
		mov	temp, a		; save adjusted high byte

		mov	index, #data_buf	; pointer to data buffer
		mov	kount, data_len		; byte counter
	vv1:
		mov	a, temp		; send adjusted high byte of address
		call	shout		;
		mov	a, r2		; send low byte of address
		call	shout		;

		; Read data and verify.

		call	shin		; read data
		mov	b, @index	; get record data
		cjne	a, b, vv2	; jump on verify fail

		jmp	vv3		; verify OK, do next address
	vv2:
		setb	c		; set error flag
		jmp	vv5		; return
	vv3:
		; Next address.

		mov	a, r2		; get low byte of address
		add	a, #1		; increment low byte
		mov	r2, a		; save incremented value
		jnc	vv4		; jump if no carry out of low byte
					; carry out of low byte
		mov	a, r3		; get high byte of address
		add	a, #1		; increment high byte
		mov	r3, a		; save incremented value
		anl	a, #00011111b	; isolate five bits
		rl	a		; move five bits to top
		rl	a		;
		rl	a		;
		orl	a, #READ_CODE	; specify code write function
		mov	temp, a		; save adjusted high byte
	vv4:
		; Next data.

		inc	index		; point to next buffer location
		djnz	kount, vv1	; decrement byte count and loop
		clr	c		; clear error flag
	vv5:
		ret


shout:

	; Shift out a byte, most significant bit first.
	; SCK expected low on entry. Return with SCK low.
	; Called with data to send in A.

		push	b
		mov	b, #8		; bit counter
	x42:
		rlc	a		; move bit into CY		1
		mov	MOSI, c		; output bit			2
		nop			; enforce data setup		1
		nop			;				1
		setb	SCK		; raise clock			1
		nop			; enforce SCK high		1
		nop			;				1
		nop			;				1
		nop			;				1
		clr	SCK		; drop clock			1
		djnz	b, x42		; next bit			2

		pop	b
		ret


shin:

	; Shift in a byte, most significant bit first.
	; SCK expected low on entry. Return with SCK low.
	; Returns received data byte in A.

		push	b
		mov	b, #8		; bit counter
	x43:
		setb	SCK		; raise clock			1
		mov	c, MISO		; input bit			1
		rlc	a		; move bit into byte		1
		nop			; enforce SCK high		1
		nop			;				1
		clr	SCK		; drop clock			1
		nop			; enforce SCK low		1
		nop			;				1
		djnz	b, x43		; next bit			2

		pop	b
		ret


erase_chip:

	; Erase target AT89S8252.

		setb	RST		; force target into reset

		mov	a, #ERASE_1	; send first byte of erase function
		call	shout		;
		mov	a, #ERASE_2	; send second byte
		call	shout		;
		mov	a, #DUMMY	; send third byte
		call	shout		;

		mov	a, #10		; wait ten milliseconds
		call	delay_ms	;

		clr	RST		; remove reset from target
		ret


shutdown:

	; Force target to abandon execution of its internal program.

		clr	SHUTDN_		; notify target of impending reset

		mov	a, #5		; give target five mS to shut down
		call	delay_ms	;

		setb	SHUTDN_		; deassert interrupt
		ret


set_pgm:

	; Prepare the target AT89S8252 for programming.

		setb	RST		; force target into reset

		mov	a, #1		; wait one millisecond (arbitrary)
		call	delay_ms	;

		; Enable writes to code and data memory.

		mov	a, #ENABLE_1	; send first byte of enable code
		call	shout		;
		mov	a, #ENABLE_2	; send second byte
		call	shout		;
		mov	a, #DUMMY	; send third byte
		call	shout		;
		ret


clear_pgm:

	; Allow target AT89S8252 to resume execution of its own program.

		clr	RST		; remove reset from target
		ret


init_longtimer0:

	; Load and start long timer zero.
	; System Timer Zero count loaded and interrupt enabled elsewhere.

		setb	LT0F		; disable counter
		setb	RS0		; select register bank one
		mov	lt0_lo, #0c8h	; load 30 second count
		mov	lt0_hi, #1	;
		clr	RS0		; back to bank zero
		clr	LT0F		; enable counter
		ret


init_longtimer1:

	; Load and start long timer one.
	; System Timer Zero count loaded and interrupt enabled elsewhere.

		setb	LT1F		; disable counter
		setb	RS0		; select register bank one
		mov	lt1, #17	; load one second count
		clr	RS0		; back to bank zero
		clr	LT1F		; enable counter
		ret


delay_ms:

	; Delay for one millisecond times the value in the accumulator.

		push	acc
		push	b

		mov	b, #0
	dd:
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	b, $		; 500 uS @ 12 MHz
		djnz	acc, dd

		pop	b
		pop	acc
		ret


delay_sec:

	; Delay for one second times the value in the accumulator.

		push	acc
		push	b

		mov	b, a
	ddd:
		mov	a, #250
		call	delay_ms	; 250 mS
		call	delay_ms	; 500 mS
		call	delay_ms	; 750 mS
		call	delay_ms	; 1000 mS

		djnz	b, ddd

		pop	b
		pop	acc
		ret


		END
